//
//  UsersTableViewController.swift
//  visionos_test_demo
//
//  Created by 3tee on 2024/7/30.
//

import UIKit
import rtc_sdk_vision

class UsersTableViewController: UITableViewController {
    var userManager: AVDUserManager?

    override func viewDidLoad() {
        super.viewDidLoad()
        self.tableView.register(UITableViewCell.self, forCellReuseIdentifier: "reuseIdentifier")
        // Uncomment the following line to preserve selection between presentations
        // self.clearsSelectionOnViewWillAppear = false

        // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
        // self.navigationItem.rightBarButtonItem = self.editButtonItem
        NotificationCenter.default.addObserver(self, selector: #selector(updateList), name: Notification.Name(NotificationName.updateList), object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(clearList), name: Notification.Name(NotificationName.clearList), object: nil)

    }
    
    deinit {
        NotificationCenter.default.removeObserver(self)
    }
    
    @objc func clearList() {
        print("接收到清空用户列表消息通知")
        DispatchQueue.main.asyncAfter(deadline: .now() + 1, execute: DispatchWorkItem(block: {
            self.tableView.reloadData()
        }))
    }
    
    @objc func updateList() {
        print("接收到更新用户列表消息通知")
        if let roomid = UserDefaults().string(forKey:K.roomId) {
           let mroom = AVDRoom.obtain(roomid)
            userManager = AVDUserManager.getUserManager(mroom)
            userManager?.delegate = self
            self.tableView.reloadData()
        }
    }

    
    // MARK: - Table view data source

    override func numberOfSections(in tableView: UITableView) -> Int {
        // #warning Incomplete implementation, return the number of sections
        return 1
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        // #warning Incomplete implementation, return the number of rows
        if let userM = userManager {
            if let arr = userM.participants {
                return arr.count
            }else{
                return 0
            }
        }else{
            return 0
        }
    }

    
    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "reuseIdentifier", for: indexPath)
        cell.textLabel?.font = UIFont.systemFont(ofSize: 30)
        if let userM = userManager {
            let user = userM.participants[indexPath.row] as? AVDUser
            cell.textLabel?.text = user?.userName
        }else{
            cell.textLabel?.text = "testShow"
        }
      
        // Configure the cell...

        return cell
    }
    

    /*
    // Override to support conditional editing of the table view.
    override func tableView(_ tableView: UITableView, canEditRowAt indexPath: IndexPath) -> Bool {
        // Return false if you do not want the specified item to be editable.
        return true
    }
    */

    /*
    // Override to support editing the table view.
    override func tableView(_ tableView: UITableView, commit editingStyle: UITableViewCell.EditingStyle, forRowAt indexPath: IndexPath) {
        if editingStyle == .delete {
            // Delete the row from the data source
            tableView.deleteRows(at: [indexPath], with: .fade)
        } else if editingStyle == .insert {
            // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
        }    
    }
    */

    /*
    // Override to support rearranging the table view.
    override func tableView(_ tableView: UITableView, moveRowAt fromIndexPath: IndexPath, to: IndexPath) {

    }
    */

    /*
    // Override to support conditional rearranging of the table view.
    override func tableView(_ tableView: UITableView, canMoveRowAt indexPath: IndexPath) -> Bool {
        // Return false if you do not want the item to be re-orderable.
        return true
    }
    */

    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destination.
        // Pass the selected object to the new view controller.
    }
    */
    
}



extension UsersTableViewController: AVDUserManagerDelegate{
    func onUserLeaveNotify(_ user: AVDUser!) {
        self.tableView.reloadData()
    }
    
    func onUserLeaveNotify(_ reason: AVDResult, leave user: AVDUser!) {
        self.tableView.reloadData()
    }
    
    func onUserUpdateNotify(_ user: AVDUser!) {
        
    }
    
    func onUserStatusNotify(_ status: AVDUserStatus!, fromUser fromId: String!) {
        
    }
    
    func onUserDataNotify(_ userData: String!, fromUser fromId: String!) {
        
    }
    
    func onUserJoinNotify(_ user: AVDUser!) {
        self.tableView.reloadData()
    }
}
